% TEST_MPOLY_SYMBOLIC -- test the subroutines in the @mpoly directory for symbolic matrices

% Copyright (c) 2004 by Fritz Keinert (keinert@iastate.edu),
% Dept. of Mathematics, Iowa State University, Ames, IA 50011.
% This software may be freely used and distributed for non-commercial
% purposes, provided this copyright statement is preserved, and
% appropriate credit for its use is given.
%
% Last update: Feb 20, 2004

disp('testing routines in mw/@mpoly for symbolic matrices');
disp(' ');

% test mpoly
P2 = [112,122;212,222];
P3 = [113,123;213,223];
P23 = cat(3,P2,P3);
P = mpoly({P2,P3},2);
Q = mpoly({P2,P3});
PP = mpoly(P23,2,'polyphase',2,1);
P11 = mpoly({112,113},2);
N = sym(P);
NN = mpoly(sym(P23),2,'polyphase',2,1);
N11 = sym(P11);
syms a b c d real
S = mpoly({[a,0;0,b],eye(2)},2);
SS = mpoly({[a,0;0,b],eye(2)},2,'polyphase',2,1);
S11 = mpoly({a,1},2);
I = mpoly(eye(2));
Z = mpoly(zeros(2));
disp('OK - mpoly');

% test display
S
N
disp('OK - display');

% test trim

Nt = trim(N,[3,4]);
if (size(Nt.coef,3) ~= 2 | Nt.coef ~= cat(3,P3,zeros(2)))
    error('failed - trim');
end

St = trim(S,[3,4]);
if (size(St.coef,3) ~= 2 | St.coef ~= cat(3,P3,zeros(2)))
    error('failed - trim');
end

Npm = (eye(2) + N) - eye(2);
if (size(Npm.coef,3) ~= 2 | Npm.min ~= N.min)
    error('failed - trim');
end

Spm = (eye(2) + S) - eye(2);
if (size(Spm.coef,3) ~= 2 | Spm.min ~= S.min)
    error('failed - trim');
end
disp('OK - trim');

% test comparisons
if ((P ~= N) | (N == S) | (P == S))
    error('failed - comparisons: eq, ne');
end

% test conversion
H = symbol(NN);
if (polyphase(H) ~= NN)
    error('failed - conversion: symbol, polyphase');
end

if (double(N) ~= P)
    error('failed - conversion: double');
end

if (reshape(N,1,4) ~= mpoly(reshape(N.coef,1,4,2),2))
    error('failed - reshape');
end

if (reshape(S,1,4) ~= mpoly(reshape(S.coef,1,4,2),2))
    error('failed - reshape');
end

disp('OK - conversions: symbol, polyphase, sym, double, reshape');

% test more comparisons
if (~isconstant(sym(I)) | isconstant(N))
    error('failed - isconstant');
end

if (~isidentity(sym(I)) | isidentity(N))
    error('failed - isidentity');
end

z = mpoly(1,1);
if (~ismonomial(sym(I)*z) | ismonomial(N))
    error('failed - ismonomial');
end

if (~isnumeric(I) | isnumeric(N) | isnumeric(S))
    error('failed - isnumeric');
end

if (issymbolic(I) | issymbolic(N) | ~issymbolic(S))
    error('failed - issymbolic');
end

if (iszero(sym(I)) | ~iszero(sym(Z)))
    error('failed - iszero');
end

disp('OK - comparisons: eq, ne, isconstant, isidentity');
disp('                  ismonomial, isnumeric, issymbolic, iszero');

% test concatenation
z = mpoly(sym(1),1);
Q = [z,0;0,z];
if (Q ~= z * eye(2))
    error('failed - concatenation');
end
disp('OK - concatenation: horzcat, vertcat');

% test right-hand subscripting
if (N.min ~= 2 | S.min ~= 2)
    error('failed - subscripting P.min');
end

if (N.max ~= 3 | S.max ~= 3)
    error('failed - subscripting P.max');
end

Scoef = [a,0;0,b];
Scoef(:,:,2) = sym(eye(2));
if (N.coef ~= P23 | S.coef ~= Scoef)
    error('failed - subscripting P.coef');
end

if (N.length ~= 2 | S.length ~= 2)
    error('failed - subscripting P.length');
end

if (N.degree ~= 1 | S.degree ~= 1)
    error('failed - subscripting P.degree');
end

if (~iszero(N.size - [2,2]) | ~iszero(S.size - [2,2]))
    error('failed - subscripting P.size');
end

if (~strcmp(NN.type,'polyphase') | ~strcmp(SS.type,'polyphase'))
    error('failed - subscripting P.type');
end

if (NN.m ~= 2 | SS.m ~= 2)
    error('failed - subscripting P.m');
end

if (NN.r ~= 1 | SS.r ~= 1)
    error('failed - subscripting P.r');
end

if (~iszero(N{end} - P3) | ~iszero(S{end} - eye(2)))
    error('failed - subscripting P{i}');
end

if (~iszero(N{4} - zeros(2)) | ~iszero(S{4} - zeros(2)))
    error('failed - subscripting P{i}');
end

if (N(1,1) ~= N11 | S(1,1) ~= S11)
    error('failed - subscripting P(i,j)');
end

disp('OK - right-hand subscripting: get, P.field, P(i,j), P{i}')

% test left-hand subscripting

Q = N;
Q.min = Q.min + 1;
if (Q ~= P*z)
    error('failed - left-hand subscripting P.min');
end

Q = S;
Q.min = Q.min + 1;
if (Q ~= S*z)
    error('failed - left-hand subscripting P.min');
end

Q = N;
Q{2} = zeros(2);
if (Q ~= mpoly(P3,3))
    error('failed - left-hand subscripting P{i}');
end

Q = S;
Q{2} = zeros(2);
if (Q ~= mpoly(eye(2),3))
    error('failed - left-hand subscripting P{i}');
end

Q = N;
Q(1,1) = z^2;
if (Q(1,1) ~= z^2)
    error('failed - left-hand subscripting P(i,j)');
end

Q = S;
Q(1,1) = z^2;
if (Q(1,1) ~= z^2)
    error('failed - left-hand subscripting P(i,j)');
end
disp('OK - left-hand subscripting: set, P.field, P(i,j), P{i}')

% test size
if (iszero(N.size - size(N)) & iszero(S.size - size(S)))
    disp('OK - size')
else
    error('failed - size');
end

% test addition/subtraction/scalar multiplication

if (N ~= +N | S ~= +S)
    error('failed - unary plus');
end

if (N + P ~= 2*P)
    error('failed - addition or scalar multiplication');
end

if (S + S ~= 2*S)
    error('failed - addition or scalar multiplication');
end

if (~iszero(-N + N))
    error('failed - unary minus');
end

if (~iszero(-S + S))
    error('failed - unary minus');
end

Iz = mpoly(eye(2),1);
if (N + Iz - N ~= Iz)
    error('failed - addition or subtraction');
end

if (S + Iz - S ~= Iz)
    error('failed - addition or subtraction');
end
disp('OK - addition, subtraction, scalar multiplication');

% test elementwise multiplication/division/power

Two = 2 * mpoly(ones(2,2,2),2);

if (N .* Two ~= N * 2 | S .* Two ~= S * 2)
    error('failed - times');
end

if (Two .\ N ~= 2 \ N | Two .\ S ~= 2 \ S)
    error('failed - ldivide');
end

if (N ./ Two ~= N / 2 | S ./ Two ~= S / 2)
    error('failed - rdivide');
end

if (N .^ 2 ~= mpoly(P23.^2,2) | S.^2 ~= mpoly({[a,0;0,b].^2,eye(2)},2))
    error('failed - power');
end
disp('OK - pointwise operations: times, ldivide, rdivide, power');

% test multiplication/division/power

disp(' ');
disp('testing division');
disp('this is split into multiple parts, and will take a while');
disp(' ');

% matrix with nonsingular leading term
L = eye(2);
L(1,1,2) = 1;
L = mpoly(L);

% case 1: L numeric
NL = N*L;
[Q,method] = mrdivide(NL,L);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

LN = L*N;
[Q,method] = mldivide(L,LN);
if (Q ~= P)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

SL = S*L;
[Q,method] = mrdivide(SL,L);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

LS = L*S;
[Q,method] = mldivide(L,LS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

disp('OK - multiplication/division: method 1 part 1, numeric');


% case 2: L symbolic constant
L = sym(L);
NL = N*L;
[Q,method] = mrdivide(NL,L);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

LN = L*N;
[Q,method] = mldivide(L,LN);
if (Q ~= P)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

SL = S*L;
[Q,method] = mrdivide(SL,L);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

LS = L*S;
[Q,method] = mldivide(L,LS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

disp('OK - multiplication/division: method 1 part 1, symbolic constant');

% case 3: L symbolic variable
L(1,1) = L(1,1) + a;
NL = N*L;
[Q,method] = mrdivide(NL,L);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

LN = L*N;
[Q,method] = mldivide(L,LN);
if (Q ~= P)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

SL = S*L;
[Q,method] = mrdivide(SL,L);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

LS = L*S;
[Q,method] = mldivide(L,LS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

disp('OK - multiplication/division: method 1 part 1, symbolic variable');

% matrix with nonsingular highest term
H = [1,0;0,0];
H(:,:,2) = eye(2);
H = mpoly(H);

% case 1: H numeric
NH = N*H;
[Q,method] = mrdivide(NH,H);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

HN = H*N;
[Q,method] = mldivide(H,HN);
if (Q ~= P)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

SH = S*H;
[Q,method] = mrdivide(SH,H);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

HS = H*S;
[Q,method] = mldivide(H,HS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

disp('OK - multiplication/division: method 1 part 2, numeric');

% case 2: H symbolic constant
H = sym(H);
NH = N*H;
[Q,method] = mrdivide(NH,H);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

HN = H*N;
[Q,method] = mldivide(H,HN);
if (Q ~= P)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

SH = S*H;
[Q,method] = mrdivide(SH,H);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

HS = H*S;
[Q,method] = mldivide(H,HS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

disp('OK - multiplication/division: method 1 part 2, symbolic constant');

% case 3: H symbolic variable
H(1,1) = H(1,1) + mpoly(a,1);
NH = N*H;
[Q,method] = mrdivide(NH,H);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

HN = H*N;
[Q,method] = mldivide(H,HN);
if (Q ~= P)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

SH = S*H;
[Q,method] = mrdivide(SH,H);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

HS = H*S;
[Q,method] = mldivide(H,HS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 1)
    warning('this should have been method 1');
end

disp('OK - multiplication/division: method 1 part 2, symbolic variable');

% projection factor
% case 1: F numeric
U = [3/5;4/5];
F = mpoly({eye(size(U,1)) - U*U',U*U'},0,'polyphase',2,1);

NF = N*F;
[Q,method] = mrdivide(NF,F);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

SF = S*F;
[Q,method] = mrdivide(SF,F);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

FN = F*N;
[Q,method] = mldivide(F,FN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

FS = F*S;
[Q,method] = mldivide(F,FS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

disp('OK - multiplication/division: method 2, numeric');

% case 2: F symbolic constant
F = sym(F);

NF = N*F;
[Q,method] = mrdivide(NF,F);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

SF = S*F;
[Q,method] = mrdivide(SF,F);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

FN = F*N;
[Q,method] = mldivide(F,FN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

FS = F*S;
[Q,method] = mldivide(F,FS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

disp('OK - multiplication/division: method 2, symbolic constant');

% case 3: F symbolic variable
U = [a;b]/sqrt(a^2+b^2);
F = mpoly({eye(size(U,1)) - U*U',U*U'},0,'polyphase',2,1);
NF = N*F;
[Q,method] = mrdivide(NF,F);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

SF = S*F;
[Q,method] = mrdivide(SF,F);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

FN = F*N;
[Q,method] = mldivide(F,FN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

FS = F*S;
[Q,method] = mldivide(F,FS);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 2)
    warning('this should have been method 2');
end

disp('OK - multiplication/division: method 2, symbolic variable');

% upper triangular matrix
% I have to use a 3x3 example to force method 3
U = [1,1,1;0,1,1;0,0,0];
U(:,:,2) = [0,0,0;0,1,0;0,0,1];
U = mpoly(U);

% case 1: U numeric

R = [112,122,132;212,222,232;312,322,332];
R(:,:,2) = [113,123,133;213,223,233;313,323,333];
R = mpoly(R,2);
N = sym(R);
S = N;
S(1,1) = S(1,1)*a;

NU = N*U;
[Q,method] = mrdivide(NU,U);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

SU = S*U;
[Q,method] = mrdivide(SU,U);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

UN = U*N;
[Q,method] = mldivide(U,UN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

US = U*S;
[Q,method] = mldivide(U,US);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

disp('OK - multiplication/division: method 3 part 1, numeric');


% case 2: U symbolic constant

U = sym(U);

NU = N*U;
[Q,method] = mrdivide(NU,U);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

SU = S*U;
[Q,method] = mrdivide(SU,U);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

UN = U*N;
[Q,method] = mldivide(U,UN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

US = U*S;
[Q,method] = mldivide(U,US);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

disp('OK - multiplication/division: method 3 part 1, symbolic constant');

% case 3: U symbolic variable

U(1,1) = b;

NU = N*U;
[Q,method] = mrdivide(NU,U);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

SU = S*U;
[Q,method] = mrdivide(SU,U);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

UN = U*N;
[Q,method] = mldivide(U,UN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

US = U*S;
[Q,method] = mldivide(U,US);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

disp('OK - multiplication/division: method 3 part 1, symbolic variable');

% lower triangular matrix
% I have to use a 3x3 example to force method 3
U = [1,1,1;0,1,1;0,0,0]';
U(:,:,2) = [0,0,0;0,1,0;0,0,1]';
U = mpoly(U);

% case 1: U numeric

R = [112,122,132;212,222,232;312,322,332];
R(:,:,2) = [113,123,133;213,223,233;313,323,333];
R = mpoly(R,2);
N = sym(R);
S = N;
S(1,1) = S(1,1)*a;

NU = N*U;
[Q,method] = mrdivide(NU,U);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

SU = S*U;
[Q,method] = mrdivide(SU,U);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

UN = U*N;
[Q,method] = mldivide(U,UN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

US = U*S;
[Q,method] = mldivide(U,US);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

disp('OK - multiplication/division: method 3 part 2, numeric');

% case 2: U symbolic constant

U = sym(U);

NU = N*U;
[Q,method] = mrdivide(NU,U);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

SU = S*U;
[Q,method] = mrdivide(SU,U);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

UN = U*N;
[Q,method] = mldivide(U,UN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

US = U*S;
[Q,method] = mldivide(U,US);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

disp('OK - multiplication/division: method 3 part 2, symbolic constant');

% case 3: U symbolic variable

U(1,1) = b;

NU = N*U;
[Q,method] = mrdivide(NU,U);
if (Q ~= N)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

SU = S*U;
[Q,method] = mrdivide(SU,U);
if (Q ~= S)
    error(['failed - right multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

UN = U*N;
[Q,method] = mldivide(U,UN);
if (Q ~= N)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

US = U*S;
[Q,method] = mldivide(U,US);
if (Q ~= S)
    error(['failed - left multiplication or division, method ',num2str(method)]);
end
if (method ~= 3)
    warning('this should have been method 3');
end

disp('OK - multiplication/division: method 3 part 2, symbolic variable');

U = [1,1;0,0];
U(:,:,2) = [0,0;0,1];
U = mpoly(U,1);

Uinv = inv(U);
if (Uinv * U ~= eye(2))
    error('failed - inv');
end

Uinv = longinv(U);
if (Uinv * U ~= eye(2))
    error('failed - longinv');
end

if (P^2 ~= P*P)
    error('failed - mpower');
end

disp('OK - multiplication/division: mltimes, mldivide, mrtimes');
disp('                              mrdivide, mpower, inv, longinv');

% test transpose

Pt = mpoly({P3',P2'},-3);
Prt = mpoly({P2',P3'},2);

if (P' ~= Pt)
    error('failed - ctranspose');
end

if (P.' ~= Prt)
    error('failed - transpose');
end

if ((P').' ~= reverse(P))
    error('failed - reverse');
end

disp('OK - transpose, ctranspose, reverse');

% test rounding

if (ceil(N - 0.1) ~= N)
    error('failed - ceil');
end

if (floor(N + 0.1) ~= N)
    error('failed - floor');
end

if (fix(N * 1.000001) ~= N)
    error('failed - fix');
end

if (round(N * 1.000001) ~= N)
    error('failed - round');
end

disp('OK - rounding: ceil, floor, round, fix');

% test diagonal/triangle

if (N ~= (diag(N) + tril(N,-1) + triu(N,1)))
    error('failed - diag, tril or triu');
end

if (S ~= (diag(S) + tril(S,-1) + triu(S,1)))
    error('failed - diag, tril or triu');
end

disp('OK - diag, tril, triu');

% test determinant

U = [a;b]/sqrt(a^2+b^2);
F = mpoly({eye(size(U,1)) - U*U',U*U'},0,'polyphase',2,1);
if (det(F) ~= z)
    error('failed - det')
end

disp('OK - det');

% test moment
if (moment(P) ~= P2+P3 | moment(P,1) ~= 2*P2+3*P3)
    error('failed - moment');
end
disp('OK - moment');

% test match_type
Q = P * PP;
R = PP * symbol(PP);
if (~strcmp(Q.type,'polyphase') | ~strcmp(R.type,''))
    error('failed - match_type');
end
disp('OK - match_type');

% test kron
Q = kron(P,P);
if (Q(3:4,1:2) ~= P(2,1)*P)
    error('failed - kron');
end
disp('OK - kron');
